from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple

from .fieldsets import get_phq9_fieldsets


def get_phq9_radio_fields() -> dict:
    return {
        # "crf_status": admin.VERTICAL,
        "ph9appetit": admin.VERTICAL,
        "ph9badabt": admin.VERTICAL,
        "ph9concen": admin.VERTICAL,
        "ph9feel": admin.VERTICAL,
        "ph9functio": admin.VERTICAL,
        "ph9interst": admin.VERTICAL,
        "ph9moving": admin.VERTICAL,
        "ph9tired": admin.VERTICAL,
        "ph9troubl": admin.VERTICAL,
        "ph9though": admin.VERTICAL,
    }


class Phq9ModelAdminMixin:
    form = None  # PatientHealthForm

    fieldsets = (
        get_phq9_fieldsets(),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = get_phq9_radio_fields()



from edc_auth.site_auths import site_auths

from .auth_objects import PHQ9, PHQ9_SUPER, PHQ9_VIEW, phq9_codenames

site_auths.add_group(*phq9_codenames, name=PHQ9_VIEW, view_only=True)
site_auths.add_group(*phq9_codenames, name=PHQ9, no_delete=True)
site_auths.add_group(*phq9_codenames, name=PHQ9_SUPER)



from django.apps import apps as django_apps
from django.conf import settings
from django.db import models

# don't delete. so attr is searchable
EDC_PHQ9_MODEL = "EDC_PHQ9_MODEL"


def get_phq9_model_name() -> str:
    return getattr(settings, "EDC_PHQ9_MODEL", "edc_phq9.phq9")


def get_phq9_model_cls() -> models.Model:
    return django_apps.get_model(get_phq9_model_name())



from django.utils.html import format_html


def get_phq9_fields() -> tuple:
    return (
        "ph9interst",
        "ph9feel",
        "ph9troubl",
        "ph9tired",
        "ph9appetit",
        "ph9badabt",
        "ph9concen",
        "ph9moving",
        "ph9though",
        "ph9functio",
    )


def get_phq9_fieldsets() -> tuple:
    return (
        "PHQ-9",
        {
            "description": format_html(
                "<h3>Over the last 2 weeks, how often have you been bothered "
                "by any of the following?</h3>"
            ),
            "fields": get_phq9_fields(),
        },
    )



from django.test import TestCase
from edc_constants.constants import NO, YES
from edc_utils import get_utcnow

from edc_phq9.constants import NEARLY_EVERYDAY
from edc_phq9.fieldsets import get_phq9_fields
from edc_phq9.forms import Phq9Form
from edc_phq9.utils import get_phq9_model_cls


class Phq9Tests(TestCase):
    def test_model(self):
        get_phq9_model_cls()

    def test_form(self):
        form = Phq9Form(data={})
        form.is_valid()
        self.assertIn("subject_identifier", form._errors)

    def test_form_performed(self):
        data = {fld: NEARLY_EVERYDAY for fld in get_phq9_fields()}
        data.update(
            subject_identifier="12345", report_datetime=get_utcnow(), ph9_performed=YES
        )
        form = Phq9Form(data=data)
        form.is_valid()
        self.assertEqual(form._errors, {})

    def test_form_not_performed(self):
        data = {fld: NEARLY_EVERYDAY for fld in get_phq9_fields()}
        data.update(subject_identifier="12345", report_datetime=get_utcnow(), ph9_performed=NO)
        form = Phq9Form(data=data)
        form.is_valid()
        self.assertIn("ph9_not_performed_reason", form._errors)
        data.update(ph9_not_performed_reason="blah blah")
        form = Phq9Form(data=data)
        form.is_valid()
        self.assertIn("ph9interst", form._errors)



from django import forms
from django.conf import settings
from edc_constants.constants import NO, YES
from edc_crf.crf_form_validator_mixins import CrfFormValidatorMixin
from edc_form_validators import FormValidatorMixin
from edc_form_validators.form_validator import FormValidator
from edc_visit_schedule.utils import raise_if_not_baseline

from edc_phq9.fieldsets import get_phq9_fields

from .utils import get_phq9_model_cls


class Phq9FormValidatorMixin:
    def clean_phq9(self):
        if getattr(settings, "EDC_PHQ9_MODEL_AT_BASELINE_ONLY", True):
            raise_if_not_baseline(self.cleaned_data.get("subject_visit"))
        self.required_if(NO, field="ph9_performed", field_required="ph9_not_performed_reason")
        for fld in get_phq9_fields():
            self.applicable_if(YES, field="ph9_performed", field_applicable=fld)


class Phq9FormValidator(Phq9FormValidatorMixin, CrfFormValidatorMixin, FormValidator):
    def clean(self):
        self.clean_phq9()


class Phq9Form(FormValidatorMixin, forms.ModelForm):
    form_validator_cls = Phq9FormValidator

    class Meta:
        model = get_phq9_model_cls()
        fields = "__all__"
